#
# oplfm: OPL2/OPL3 Instrument Toolkit for C and Python
# Copyright (C) 2016, 2017 Thomas Perl <m@thp.io>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#

import oplfm
import time
import math

instruments = [oplfm.Instrument(name) for name in ('BASS1', 'BASS2', 'CLARINET', 'ELPIANO2', 'OBOE1',
                                                  'ORGAN1', 'STRINGS1', 'VIBRA2', 'XYLO1',)]

note = 30
delta = 7
idx = 0
while True:
    instr = instruments[idx % len(instruments)]
    note += delta
    if note > 90 or note < 30:
        delta = (-5 if delta > 0 else 7)
        note = max(30, min(90, note))
        idx += 1

    for velocity in range(20, 127, 30):
        instr.note(note, 0)
        time.sleep(0.01)
        instr.note(note, velocity)

    time.sleep(0.2 * abs(math.sin(note * 0.01)))

    instr.note(note, 0)



# For this example, you need to install "mido" (pip3 install mido)
# and obtain a copy of "canyon.mid" from Windows 3.x or 9x.
# It ignores most instruments and MIDI commands, but just plays
# the voice and bass on the OPL with basic instruments.

import mido
import oplfm

organ = oplfm.Instrument('PIANO1')
bass = oplfm.Instrument('BASS2')

for msg in mido.MidiFile('canyon.mid').play():
    print(msg)
    if msg.type in ('note_on', 'note_off'):
        if msg.channel == 1:
            organ.note(msg.note, msg.velocity)
        elif msg.channel == 2:
            bass.note(msg.note, msg.velocity)



#
# oplfm: OPL2/OPL3 Instrument Toolkit for C and Python
# Copyright (C) 2016, 2017 Thomas Perl <m@thp.io>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#

import oplfm
import time
import sys

i = {
    'piano':   oplfm.Instrument('PIANO1'),
    'bdrum': oplfm.Instrument('BDRUM1'),
    'hihat': oplfm.Instrument('NOISE1'),
}

notes = {
    'bdrum': 50,
    'hihat': 60,
    'piano': 20,
}

rhythm = [
    {'bdrum': 'x...x...x...x.x.x...x...x...x.x.x...x...x...x.x.x.x.x.x.x.x.x.x.',
     'hihat': '..................x...x...........x...x.........................',
     'piano': '................x...............x...........x.......x.......x...',},

    {'bdrum': 'x...x...x...x.x.x...x...x...x.x.x...x...x...x...x...x...x...x...',
     'hihat': '..................x...x...........x...x.....x.x.x.x.x.xxxxxxxxxx',
     'piano': '................x...............................................',},

    {'bdrum': 'x...x...x...x.x.x...x...x...x.x.x...x.......x.......x.......x...',
     'hihat': '.......................................xx......xx......xx.......',
     'piano': '.x...x...x.x.x...x...x...x.x.x..................................',},
]

delay = 0.05

while True:
    for row in rhythm:
        for index in range(max(len(pattern) for pattern in row.values())):
            if index % 4 == 0:
                print()

            for instrument in sorted(row.keys()):
                inst = i[instrument]
                c = row[instrument][index]
                sys.stdout.write(instrument[0] if c == 'x' else '-')
                if c == 'x':
                    inst.note(notes[instrument], 127)

            time.sleep(delay)

            for instrument in sorted(row.keys()):
                inst = i[instrument]
                c = row[instrument][index]
                if c == 'x':
                    inst.note(notes[instrument], 0)

            time.sleep(delay)
            sys.stdout.write(' ')
            sys.stdout.flush()



#
# oplfm: OPL2/OPL3 Instrument Toolkit for C and Python
# Copyright (C) 2016, 2017 Thomas Perl <m@thp.io>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#

import oplfm
import time

piano = oplfm.Instrument('PIANO2')
bass = oplfm.Instrument('HARP1')

C1, D1, E1, F1, G1, A1, B1 = 24, 26, 28, 27, 31, 33, 35

OCTAVE = 12

OFF3 = E1 - C1
OFF5 = G1 - C1

CHORDS = [C1, F1, G1, C1, G1, F1, C1, F1]

MELODY = [
    [   0, OFF3, OFF5, OFF3, OFF5,    0, OFF5,    0],
    [OFF3, OFF3, OFF3, OFF3, OFF5, OFF3, OFF3, OFF3],
    [OFF3, OFF5, None, None, OFF3, OFF5, None, None],
    [   0,    1,    2,    3,    4,    5,    6,    7],
    [   7,    6,    5,    4,    3,    2,    1,    0],
]

def play_chord(instrument, base, velocity):
    base += 2 * OCTAVE
    instrument.note(base, velocity)
    instrument.note(base + OFF3, velocity)
    instrument.note(base + OFF5, velocity)


def play_melody(instrument, note, offset, velocity, delay_duration, press_duration, release_duration):
    time.sleep(delay_duration)
    if note is not None:
        instrument.note(note + offset, velocity)
    time.sleep(press_duration)
    if note is not None:
        instrument.note(note + offset, 0)
    time.sleep(release_duration)

while True:
    for i, chord in enumerate(CHORDS):
        print('='*chord)
        play_chord(bass, chord, 127)
        for j, note in enumerate(MELODY[i%len(MELODY)]):
            print('-'*(note or 0))
            play_melody(piano, note, chord + 3 * OCTAVE, 100, 0.01, 0.1, 0.02)
        play_chord(bass, chord, 0)



#
# oplfm: OPL2/OPL3 Instrument Toolkit for C and Python
# Copyright (C) 2016, 2017 Thomas Perl <m@thp.io>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#

import oplfm
import time

moog = oplfm.Instrument('MOOGSYNT')
organ = oplfm.Instrument('PIANO1')
bass = oplfm.Instrument('BASS2')

pattern0 = [ 60, 70, 60, 68, 60, 66, 60, 64 ]
pattern1 = [ 60, 60, 70, 70, 65, 65, 55, 55 ]
pattern2 = [ 60, 70, 60, 70, 60, 70, 66, 62 ]
pattern3 = [ 60, 50, 55, 45, 50, 40, 45, 35 ]

sequence = [
    pattern0, pattern0, pattern0, pattern1,
    pattern0, pattern2, pattern0, pattern3,
    pattern0, pattern1, pattern2, pattern3,
    pattern3, pattern3, pattern3, pattern1,
]

while True:
    for pattern in sequence:
        for note in pattern:
            moog.note(note, 127)
            bass.note(int(20+note/2), 120)
            time.sleep(.01)

            organ.note(note+5, 100)
            time.sleep(.4)

            bass.note(int(20+note/2), 0)
            organ.note(note+5, 0)
            moog.note(note, 0)
            time.sleep(.01)



from distutils.core import setup
from distutils.extension import Extension

SOURCES = [
    'src/oplfm_python.c',
    'src/oplfm.c', 'src/instrument.c', 'src/playback.c', 'src/stream.c',
    'src/woody/woodyopl.cpp', 'src/woody/woody.cpp',
]

setup(name='oplfm', version='1.1', description='OPL2/OPL3 Instrument Toolkit', license='GPL',
      author='Thomas Perl', author_email='m@thp.io',
      url='https://github.com/thp/oplfm',
      ext_modules=[Extension('oplfm', SOURCES, include_dirs=['include'], libraries=['portaudio'])])
